package com.fs.qw.service.impl;


import com.fs.common.utils.DateUtils;
import com.fs.qw.domain.QwMaterial;
import com.fs.qw.mapper.QwMaterialMapper;
import com.fs.qw.param.QwMaterialParam;
import com.fs.qw.service.IQwMaterialService;
import com.fs.qw.vo.QwMaterialVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 素材库Service业务层处理
 *
 * @author fs
 * @date 2024-06-20
 */
@Service
public class QwMaterialServiceImpl implements IQwMaterialService
{
    @Autowired
    private QwMaterialMapper qwMaterialMapper;

    /**
     * 查询素材库
     *
     * @param materialId 素材库主键
     * @return 素材库
     */
    @Override
    public QwMaterial selectQwMaterialByMaterialId(Long materialId)
    {
        return qwMaterialMapper.selectQwMaterialByMaterialId(materialId);
    }

    /**
     * 查询素材库列表
     *
     * @param param 素材库
     * @return 素材库
     */
    @Override
    public List<QwMaterialVO> selectQwMaterialList(QwMaterialParam param)
    {
        return qwMaterialMapper.selectQwMaterialListVO(param);
    }

    /**
     * 新增素材库
     *
     * @param qwMaterial 素材库
     * @return 结果
     */
    @Override
    public int insertQwMaterial(QwMaterial qwMaterial)
    {
        qwMaterial.setCreateTime(DateUtils.getNowDate());
        return qwMaterialMapper.insertQwMaterial(qwMaterial);
    }

    /**
     * 修改素材库
     *
     * @param qwMaterial 素材库
     * @return 结果
     */
    @Override
    public int updateQwMaterial(QwMaterial qwMaterial)
    {
        qwMaterial.setUpdateTime(DateUtils.getNowDate());
        return qwMaterialMapper.updateQwMaterial(qwMaterial);
    }

    /**
     * 批量删除素材库
     *
     * @param materialIds 需要删除的素材库主键
     * @return 结果
     */
    @Override
    public int deleteQwMaterialByMaterialIds(Long[] materialIds)
    {
        return qwMaterialMapper.deleteQwMaterialByMaterialIds(materialIds);
    }

    /**
     * 删除素材库信息
     *
     * @param materialId 素材库主键
     * @return 结果
     */
    @Override
    public int deleteQwMaterialByMaterialId(Long materialId)
    {
        return qwMaterialMapper.deleteQwMaterialByMaterialId(materialId);
    }
}
