package com.fs.sop.mapper;

import com.fs.common.annotation.DataSource;
import com.fs.common.enums.DataSourceType;
import com.fs.sop.domain.SopUserLogsInfo;
import com.fs.sop.params.BatchSopUserLogsInfoParamU;
import com.fs.sop.params.DeleteQwSopParam;
import com.fs.sop.params.SopUserLogsInfoDelParam;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface SopUserLogsInfoMapper {
    /**
     * 插入一条记录
     * @param sopUserLogsInfo 实体对象
     * @return 影响行数
     */
    @DataSource(DataSourceType.SOP)
    int insertSopUserLogsInfo(SopUserLogsInfo sopUserLogsInfo);

    /**
     * 插入一条记录
     * @param sopUserLogsInfo 实体对象
     * @return 影响行数
     */
    @DataSource(DataSourceType.SOP)
    int insertChatSopUserLogsInfo(SopUserLogsInfo sopUserLogsInfo);

    /**
     * 根据ID查询记录
     * @param id 主键ID
     * @return 查询结果
     */
    @DataSource(DataSourceType.SOP)
    SopUserLogsInfo selectById(String id);

    /**
     * 查询所有记录
     * @return 记录列表
     */
    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectAll();

    /**
     * 根据ID删除记录
     * @param id 主键ID
     * @return 影响行数
     */
    @DataSource(DataSourceType.SOP)
    int deleteById(String id);

    @DataSource(DataSourceType.SOP)
    int deleteSopUserLogsInfoByIds(String[] ids);

    /**
    * 根据员工id删除
    */
    @DataSource(DataSourceType.SOP)
    int deleteByQwUserIdAndCorpIdToContactId(@Param("qwUserId") String qwUserId, @Param("corpId") String corpId, @Param("externalContactId") String externalContactId);


    @DataSource(DataSourceType.SOP)
    int deleteByQwUserIdAndCorpIdToContactIdByChat(@Param("qwUserId") String qwUserId, @Param("corpId") String corpId, @Param("externalContactId") String externalContactId);


    /**
     * 根据员工id删除
     */
    @DataSource(DataSourceType.SOP)
    int deleteByQwUserIdAndCorpIdToContactIdList(@Param("data") DeleteQwSopParam param);


    @DataSource(DataSourceType.SOP)
    int deleteByQwUserIdAndCorpIdToContactIdByChatList(@Param("data") DeleteQwSopParam param);


    /**
     * 根据sop任务-标签-删除对应的客户的营期-
     */
    @DataSource(DataSourceType.SOP)
    int deleteBySopIdToContactId(@Param("data") SopUserLogsInfoDelParam logsInfoDelParam);

    @DataSource(DataSourceType.SOP)
    int deleteBySopIdToContactIdChat(@Param("data") SopUserLogsInfoDelParam logsInfoDelParam);


    /**
    * 批量删除
    */
    @DataSource(DataSourceType.SOP)
    int deleteSopUserLogsInfoBySopIds(@Param("sopIds") String[] sopIds);


    /**
     * 根据ID修改记录
     * @return 影响行数
     */
    @DataSource(DataSourceType.SOP)
    int updateById(SopUserLogsInfo info);

    /**
     * 查询某个SOP的任务的营期的客户列表
     * @return 记录列表
     */
    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectSopUserLogsInfoList(SopUserLogsInfo info);


    @DataSource(DataSourceType.SOP)
    SopUserLogsInfo selectSopUserLogsInfo(SopUserLogsInfo info);

    @DataSource(DataSourceType.SOP)
    SopUserLogsInfo selectSopUserLogsInfoByCheckJobTransfer(SopUserLogsInfo info);

    @DataSource(DataSourceType.SOP)
    SopUserLogsInfo selectChatSopUserLogsInfo(SopUserLogsInfo info);

    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectSopUserLogsInfoListBySopId(@Param("sopId") String sopId, @Param("userLogsId") String userLogsId);


    @DataSource(DataSourceType.SOP)
    void batchInsertSopUserLogsInfo(@Param("SopUserLogsInfo") List<SopUserLogsInfo> logsToInsert);

    @DataSource(DataSourceType.SOP)
    void batchUpdateSopUserLogsInfoToTime(BatchSopUserLogsInfoParamU paramU);

    /**
     * 查询sopUserLogsInfo
     *
     * @param id sopUserLogsInfo主键
     * @return sopUserLogsInfo
     */
    @DataSource(DataSourceType.SOP)
    public SopUserLogsInfo selectSopUserLogsInfoById(String id);

    @DataSource(DataSourceType.SOP)
    public List<SopUserLogsInfo> selectSopUserLogsInfoByIds(@Param("ids") String[] ids);

    /**
     * 修改sopUserLogsInfo
     *
     * @param sopUserLogsInfo sopUserLogsInfo
     * @return 结果
     */
    @DataSource(DataSourceType.SOP)
    public int updateSopUserLogsInfoToTime(@Param("data") SopUserLogsInfo sopUserLogsInfo);

    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectChatSopUserLogsByQwUserId(@Param("qwUserId")String qwUserId,@Param("corpId") String corpId);
    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectEerGroup();
    @DataSource(DataSourceType.SOP)
    List<SopUserLogsInfo> selectSopUserLogsInfoListByUserLogsId(String id);
}
